<?php include_once 'codebase/hyperframe.php' ?>
<h1>Advanced installation</h1>

<p>The basic installation assumes that all webpages are in the root level of the Hyperframe site, and that any new pages are created using the online tools. This arrangement has been chosen in v4 because it allows for an installation which in most cases will &#39;just work&#39; straight from the unzipped fileset, without the need for any adjustment of settings.</p>

<p>Note that when we say &#39;root of the site structure&#39; that refers to this Hyperframe instance, and does restrict the website to being in the domain root.&nbsp;</p>

<p>In order to achieve this, it relies upon a &#39;shebang&#39; or reflex loader command being present in the first line of each webpage. For site-root pages this will be <strong>&lt;?php include_once &#39;codebase/hyperframe.php&#39; ?&gt;</strong> -and this must literally be the very first statement in the file, before any doctype or head tags.&nbsp;</p>

<p>If you need to import existing webpages into a Hyperframe site, then the simple option for a small number of pages is to manually add the shebang to each page.</p>

<p>When importing large numbers of pages or when building a multilevel site, it may be more convenient to activate automatic reflex loading. In that case, the shebang in each page is unnecessary -saving a great deal of donkey work- and any ordinary html file should then act as a Hyperframe page.</p>

<p>The reason automatic reflex loading isn&#39;t activated by default is the myriad of differing webserver configurations found on hosting space, and the fact that an unsuitable setting may crash your website, leaving only a cryptic &#39;500 Error&#39; message in the browser. Thus, you may need to do a few relatively simple tests (or check the host company documentation) to find out what setting will work in your case.</p>

<h3>Apache module mode servers</h3>

<p>These are the more traditional form of php-capable&nbsp; In their case, the setting which controls reflex loading is in the <strong>.htaccess</strong> file, and has the form of</p>

<p><strong>php_value auto_prepend_file codebase/hyperframe.php</strong></p>

<p>This line is present already in the supplied .htaccess, and only needs the # removing to activate it. Doing so will basically have two possible outcomes; pages will load into Hyperframe without the shebang, or else you see a &#39;500 Error&#39; and absolutely nothing works. If the latter you are not on a module based server and should re-comment the line. Don&#39;t be too worried about testing this, as the &#39;500 Error&#39; will not do any permanent harm.</p>

<h3>CGI or FastCGI mode servers</h3>

<p>The more modern way to run php is in CGI mode, and most sites prefer this since it offers better security. <em>On all such sites you cannot use the .htaccess approach to reflex loading. </em>If you try that, you will see a 500 Error.</p>

<p>A further complication with CGI sites, in that there are two distinct ways in which a reflex loader command might have to be added. That is, via <strong>php.ini</strong> file, or via a <strong>.user.ini</strong> file. These two use the typical ini-file syntax for commands:</p>

<p><strong>auto_prepend_file = &#39;codebase/hyperframe.php&#39;</strong></p>

<p>On any given site it&#39;s your best guess as to which will work, and which won&#39;t. Occasionally, either is allowed. Two important behavioural differences exist between php.ini and .user.ini.</p>

<p>A php.ini file overrides ALL settings in higher directories, including those in the server&#39;s configuration itself. Thus in some cases, having a blank or one-line php.ini might cause things to misbehave. This is rare, though, as the defaults are usually OK anyway. A php.ini file only affects the webpages in its own directory/folder Therefore on a larger, multilevel website you may need a php.ini at each level with webpages.</p>

<p>A .user.ini file does not pre-empt settings made at higher levels on the server, so there is no issue with loss of the defaults set by the hosting company. Settings in .user.ini propagate down through site levels, so in principle only one is needed, in the Hyperframe site root.&nbsp; Though, if you need to countermand the settings in any given .user.ini at a lower level of the site, this may require further .user.ini files at each level.&nbsp; To allow for this situation, system locations in Hyperframe come presupplied with&nbsp; a .user.ini countermanding&nbsp; the reflex loader. Just to ensure it doesn&#39;t trigger on system pages.</p>

<p>In general there is no harm in leaving php.ini and .user.ini files in the site if they do nothing on that particular server.</p>

<p>By the way, note that <strong>.</strong>htaccess and <strong>.</strong>user.ini start with a dot, whilst php.ini has no leading dot.</p>

<h3>Multilevel sites</h3>

<p>For larger sites it may be preferable to use a fully-qualified path (from the server&#39;s filesystem root) to hyperframe.php instead of a relative one. Done this way, you only need one .htaccess or .user.ini to cover reflex loading throughout a multilevel site. The issue to watch here is that typical webserver&#39;s path structure is not the same as the path you see in the browser&#39;s URL bar. To find the filesystem path, you should either check your host&#39;s online documentation, or run a phpinifo() command on the server. An .user.ini example might be</p>

<p><strong>auto_prepend_file = &#39;/home/user/htdocs/codebase/hyperframe.php</strong>&#39;</p>

<p>-but remember this is <em>only an example</em> and almost certainly will <em>not</em> apply in your case.</p>

<p>The above may sound a little daunting but in most cases it is the work of a few minutes to find the right setting, and once done, pages can be uploaded as-is to any site level. we&#39;ve been looking at an automated installler to handle this aspect, but the sheer number of combinations, plus the fact that some will lock the installer itself out with a 500 error, make it a hard thing to automate any tests. Basically, this is an area where webservers need to achieve a better level of standardisation.</p>

<h3>Standard file extensions</h3>

<p>A change in v4 is the use of .php as the default file extension. As we&#39;d like to ensure that Hyperframe works in most situations out-of-zip,&nbsp; this is to avoid any possible issues with the relatively few webservers around these days which cannot run php from within htm or .html files.</p>

<p>In practice though, for live sites it is often better to use .htm or .html as the document extension. For one thing, this makes the import of existing pages easier. Plus, if you allow extensions to be seen (see extensionless working) then most people prefer to see a familiar htm(l) extension. To allow the editor to create .htm(l) files by default, change the default_extension setting in siteini.php.</p>

<p>See the supplied .htaccess file for advice on configuring your webserver to run php from within html files, if it does not already do so.</p>

<p>&nbsp;</p>
